%===============================================================================
% Algoritma Genetika Standar (dengan grafis 2D) terdiri dari:
%
% 1. Satu populasi dengan UkPop kromosom
% 2. Binary encoding
% 3. Linear fitness ranking
% 4. Roulette-wheel selection
% 5. Pindah silang satu titik potong
% 6. Probabilitas pindah silang dan probabilitas mutasi bernilai tetap 
% 7. Elitisme, satu atau dua buah kopi dari individu bernilai fitness tertinggi
% 8. Generational replacement: mengganti semua individu dengan individu baru
%
% Tanggal update 28 Desember 2004
% Suyanto - Teknik Informatika - Sekolah Tinggi Teknologi Telkom
% Jl Telekomunikasi No 1 Dayeuh Kolot - Bandung 40257
%===============================================================================

clc                           % Me-refresh command window
clear all                     % Menghapus semua semua variabel yang sedang aktif

Nvar        = 2;              % Jumlah variabel pada fungsi yang dioptimasi
Nbit        = 10;             % Jumlah bit yang mengkodekan satu variabel
JumGen      = Nbit*Nvar;      % Jumlah gen dalam kromosom
Rb          = -5.12;          % Batas bawah interval
Ra          = 5.12;           % Batas atas interval

UkPop       = 200;            % Jumlah kromosom dalam populasi
Psilang     = 0.8;            % Probabilitas pindah silang
Pmutasi     = 0.05;           % Probabilitas mutasi
MaxG        = 100;            % Jumlah generasi

BilKecil    = 10^-1;          % Digunakan untuk menghindari pembagian dengan 0
Fthreshold  = 1/BilKecil;     % Threshold untuk nilai Fitness
Bgraf       = Fthreshold;     % Untuk menangani tampilan grafis

% Inisialisasi grafis 2D
hfig = figure;
hold on
title('Optimasi fungsi menggunakan AG standar dengan grafis 2 dimensi')
set(hfig, 'position', [50,50,600,400]);
set(hfig, 'DoubleBuffer', 'on');
axis([1 MaxG 0 Bgraf]);
hbestplot = plot(1:MaxG,zeros(1,MaxG));
htext1 = text(0.6*MaxG,0.30*Bgraf,sprintf('Fitness terbaik: %7.4f', 0.0));
htext2 = text(0.6*MaxG,0.25*Bgraf,sprintf('Variabel X1: %5.4f', 0.0));
htext3 = text(0.6*MaxG,0.20*Bgraf,sprintf('Variabel X2: %5.4f', 0.0));
htext4 = text(0.6*MaxG,0.15*Bgraf,sprintf('Nilai minimum: %5.4f', 0.0));
htext5 = text(0.6*MaxG,0.10*Bgraf,sprintf('Ukuran populasi: %3.0f', 0.0));
htext6 = text(0.6*MaxG,0.05*Bgraf,sprintf('Probabilitas mutasi: %5.4f', 0.0));
xlabel('Generasi');
ylabel('Fitness terbaik');
hold off
drawnow;

% Inisialisasi populasi
Populasi = InisialisasiPopulasi(UkPop,JumGen);

% Loop evolusi
for generasi=1:MaxG,
   x = DekodekanKromosom(Populasi(1,:),Nvar,Nbit,Ra,Rb);
   Fitness(1) = EvaluasiIndividu(x,BilKecil);
   MaxF = Fitness(1);
   MinF = Fitness(1);
   IndeksIndividuTerbaik = 1;
   for ii=2:UkPop,
     Kromosom = Populasi(ii,:);
     x = DekodekanKromosom(Kromosom,Nvar,Nbit,Ra,Rb);
     Fitness(ii) = EvaluasiIndividu(x,BilKecil);
     if (Fitness(ii) > MaxF),
        MaxF = Fitness(ii);
        IndeksIndividuTerbaik = ii;
        BestX = x;
     end
     if (Fitness(ii) < MinF),
        MinF = Fitness(ii);
     end
   end
   
   % Penanganan grafis 2D
   plotvector = get(hbestplot,'YData');
   plotvector(generasi) = MaxF;
   set(hbestplot,'YData',plotvector);
   set(htext1,'String',sprintf('Fitness terbaik: %7.4f', MaxF));
   set(htext2,'String',sprintf('Variabel X1: %5.4f', BestX(1)));
   set(htext3,'String',sprintf('Variabel X2: %5.4f', BestX(2)));
   set(htext4,'String',sprintf('Nilai minimum: %5.4f', (1/MaxF)-BilKecil));
   set(htext5,'String',sprintf('Ukuran populasi: %3.0f', UkPop));
   set(htext6,'String',sprintf('Probabilitas mutasi: %5.4f', Pmutasi));
   drawnow

   if MaxF >= Fthreshold,
      break;
   end
   
   TempPopulasi = Populasi;

   % Elitisme:
   % - Buat satu kopi kromosom terbaik jika ukuran populasi ganjil
   % - Buat dua kopi kromosom terbaik jika ukuran populasi genap
   if mod(UkPop,2)==0,           % ukuran populasi genap
      IterasiMulai = 3;
      TempPopulasi(1,:) = Populasi(IndeksIndividuTerbaik,:);
      TempPopulasi(2,:) = Populasi(IndeksIndividuTerbaik,:);
   else                          % ukuran populasi ganjil
      IterasiMulai = 2;
      TempPopulasi(1,:) = Populasi(IndeksIndividuTerbaik,:);
   end
       
   LinearFitness = LinearFitnessRanking(UkPop,Fitness,MaxF,MinF);

   % Roulette-wheel selection dan pindah silang
   for jj=IterasiMulai:2:UkPop,
       IP1 = RouletteWheel(UkPop,LinearFitness);
       IP2 = RouletteWheel(UkPop,LinearFitness);
       if (rand < Psilang),
          Anak = PindahSilang(Populasi(IP1,:),Populasi(IP2,:),JumGen);
          TempPopulasi(jj,:) = Anak(1,:);
          TempPopulasi(jj+1,:) = Anak(2,:);
        else
          TempPopulasi(jj,:) = Populasi(IP1,:);
          TempPopulasi(jj+1,:) = Populasi(IP2,:);
        end    
   end

   % Mutasi dilakukan pada semua kromosom
   for kk=IterasiMulai:UkPop,
      TempPopulasi(kk,:) = Mutasi(TempPopulasi(kk,:),JumGen,Pmutasi);
   end   

   % Generational Replacement: mengganti semua kromosom sekaligus
   Populasi = TempPopulasi;
   
end